/*----------------------------------------------------------------------------
*
* The confidential and proprietary information contained in this file may
* only be used by a person authorised under and to the extent permitted
* by a subsisting licensing agreement from ARM Limited.
*
*        (C) COPYRIGHT 2008-2009,2011-2013 ARM Limited.
*             ALL RIGHTS RESERVED
*             
* This entire notice must be reproduced on all copies of this file
* and copies of this file may only be made by a person if such person is
* permitted to do so under the terms of a subsisting license agreement
* from ARM Limited.
*
*     SVN Information
*
*     Checked In          : $Date$
*
*     Revision            : $Revision$
*
*     Release Information : r0p0-00rel0
*
*-----------------------------------------------------------------------------*/
/*
*-----------------------------------------------------------------------------
Uses a WriteValue job to write to a range of address on GPU and then reads back the written value on CPU to confirm address lines are correctly wired up.
The addresses start at {{DRAM base address} | 0x0004800}, up to the address {{DRAM base address} | 0xc004800}.
The output should look similar to the following:
    4020930 ns: Check 01/16 passed  ---> write_address=0x8004800 CC_reg=0x33fd write_value=0x33fd
    5644570 ns: Check 02/16 passed  ---> write_address=0x8005000 CC_reg=0x676e write_value=0x676e
    7264810 ns: Check 03/16 passed  ---> write_address=0x8006000 CC_reg=0x9ada write_value=0x9ada
    8888210 ns: Check 04/16 passed  ---> write_address=0x800c000 CC_reg=0xce44 write_value=0xce44
   10506810 ns: Check 05/16 passed  ---> write_address=0x800c000 CC_reg=0x10218 write_value=0x10218
   12128250 ns: Check 06/16 passed  ---> write_address=0x8014000 CC_reg=0x13525 write_value=0x13525
   13756570 ns: Check 07/16 passed  ---> write_address=0x8024000 CC_reg=0x168c8 write_value=0x168c8
   15376550 ns: Check 08/16 passed  ---> write_address=0x8044000 CC_reg=0x19c76 write_value=0x19c76
   17002730 ns: Check 09/16 passed  ---> write_address=0x8084000 CC_reg=0x1d026 write_value=0x1d026
   18625230 ns: Check 10/16 passed  ---> write_address=0x8104000 CC_reg=0x2037c write_value=0x2037c
   20255710 ns: Check 11/16 passed  ---> write_address=0x8204000 CC_reg=0x23798 write_value=0x23798
   21884710 ns: Check 12/16 passed  ---> write_address=0x8404000 CC_reg=0x26b7b write_value=0x26b7b
   23511510 ns: Check 13/16 passed  ---> write_address=0x8804000 CC_reg=0x29fa0 write_value=0x29fa0
   25143630 ns: Check 14/16 passed  ---> write_address=0x9004000 CC_reg=0x2d34c write_value=0x2d34c
   26779490 ns: Check 15/16 passed  ---> write_address=0xa004000 CC_reg=0x3072b write_value=0x3072b
   28407790 ns: Check 16/16 passed  ---> write_address=0xc004000 CC_reg=0x33ac5 write_value=0x33ac5
   
*-----------------------------------------------------------------------------
*/


#include "MaliFns.h"
#include "MaliDefines-t720.h"
#include "adv1_t720_check_0_a.h"
#include "adv1_t720_mem_0.h" 

int RunMaliTest_adv1_t720_part0       (void);
int Check_adv1_t720_check_0_a         (struct t_adv1_t720_mem_0 *p);

void adv1_delay                       (void);
int  adv1_RunWriteTest                (volatile unsigned int * write_value_descriptor__address, volatile unsigned int * write_address, int current_job_slot, int res);
unsigned int PollReg                  (unsigned int unit, unsigned int core, unsigned int regnum, unsigned int bit_location, unsigned int val, char * name_str);

static volatile struct t_adv1_t720_mem_0 *malidata_0; 
static int mali_step;
static int loop_count;
static int max_iteration;

int RunMaliTest_adv1_t720 (int init) {
  if (init) { mali_step = 0; };
  switch (mali_step) {
    case 0  : return RunMaliTest_adv1_t720_part0();
    default : return 0;
  };
};

int RunMaliTest_adv1_t720_part0 (void) {
  int res             = 0;
  int gpu_rawstat     = Mali_RdReg(0x20,0,GPU_IRQ_RAWSTAT);
  int gpu_status      = Mali_RdReg(0x20,0,GPU_IRQ_STATUS);
  int mmu_irq_rawstat = Mali_RdReg(0x20,0,MMU_IRQ_RAWSTAT);
  int gpuid, num_cores, l2_size, axi_width, i, as_present, js_present, core_bitmap;
  int j, current_job_slot;
  volatile unsigned int * write_address;
  volatile unsigned int * write_value_descriptor__address;
  volatile unsigned int * write_value_descriptor__status;
  unsigned int tmp_address, tmp_value, address_step, max_address;
  unsigned int ttb_address_offset, test_address_offset_min, test_address_offset_one_hot;
  
  printf("RUNNING TEST: adv1\n");
  printf("PURPOSE     : Tests correct access to the available system memory space\n");

  /* Get current configuration to allow testing all registers */
  gpuid         = Mali_RdReg(0x20, 0, 0x0000);
  core_bitmap   = Mali_RdReg(0x20, 0, 0x0100);
  l2_size       = ((Mali_RdReg(0x20, 0, 0x0004) >> 16) & 0xFF);
  axi_width     = (1 <<((Mali_RdReg(0x20, 0, 0x0004) >> 24) & 0xFF));
  as_present    = Mali_RdReg(0x20, 0, 0x0018);
  js_present    = Mali_RdReg(0x20, 0, 0x001c);

  i = core_bitmap;
  num_cores = 0;
  while(i) {
    num_cores++;
    i >>= 1;
  }
  printf("Selected configuration:\n");
  printf("  Bus width: %d\n", axi_width);
  printf("  Number of shader cores: %d\n", num_cores);
  printf("  L2 cache size: %d kB\n", ((1<<l2_size)/1024));

  Mali_Reset();
  
  /* ********************************** */
  printf ("Power on the shader cores\n");
  Mali_WrReg(0x20,0x0, SHADER_PWRON_LO     , 0xFFFFFFFF);
  printf ("Power on the Tiler\n");
  Mali_WrReg(0x20,0x0, TILER_PWRON_LO      , 0xFFFFFFFF);

  printf ("Poll the power changed all flag until it is set\n");
  (void) Mali_PollReg (0x20, 0x0, GPU_IRQ_RAWSTAT, 10, 1, "GPU_IRQ_RAWSTAT[POWER_CHANGED_ALL]");
  gpu_rawstat = Mali_RdReg(0x20,0,GPU_IRQ_RAWSTAT);
  printf ("GPU IRQ RAWSTAT = 0x%x\n", gpu_rawstat);
  /* ********************************** */

  /* clear IRQs and set all masks */
  Mali_clear_irqs_and_set_all_masks();
  
#ifdef DEBUG_ON
  printf (" DBG: Check that the descriptor got correctly loaded into memory\n");
#endif
  (struct t_adv1_t720_mem_0 *)malidata_0 = Mali_LdMem(&adv1_t720_mem_0,sizeof(struct t_adv1_t720_mem_0),(int)adv1_t720_mem_0.ttb);
  Check_adv1_t720_check_0_a (malidata_0);

#ifdef DEBUG_ON
  printf (" DBG: Setup the four MMU address spaces in Identity mode \n");
#endif
    Mali_write_ASn_register (ASn_TRANSTAB_LO, 0, 0x00000001);
    Mali_write_ASn_register (ASn_TRANSTAB_LO, 1, 0x00000001);
    Mali_write_ASn_register (ASn_TRANSTAB_LO, 2, 0x00000001);
    Mali_write_ASn_register (ASn_TRANSTAB_LO, 3, 0x00000001);

#ifdef DEBUG_ON
  printf (" DBG: Execute the update command for four MMU address spaces\n");
#endif
    Mali_write_ASn_register (ASn_COMMAND, 0, 0x00000001);
    Mali_write_ASn_register (ASn_COMMAND, 1, 0x00000001);
    Mali_write_ASn_register (ASn_COMMAND, 2, 0x00000001);
    Mali_write_ASn_register (ASn_COMMAND, 3, 0x00000001);

  mmu_irq_rawstat = Mali_RdReg(0x20,0,MMU_IRQ_RAWSTAT);
#ifdef DEBUG_ON
  printf (" DBG: mmu_irq_rawstat=0x%x\n", mmu_irq_rawstat);
#endif

#ifdef DEBUG_ON
  printf (" DBG: Initialise the write addresses \n");
#endif
  ttb_address_offset  = (unsigned int)adv1_t720_mem_0.ttb - MALIBASE;
  test_address_offset_min = sizeof(struct t_adv1_t720_mem_0);

  test_address_offset_one_hot = 0x1;
  while (test_address_offset_one_hot < test_address_offset_min)
  {
    test_address_offset_one_hot = test_address_offset_one_hot << 1;
  }

  write_address = ((unsigned int *) (test_address_offset_one_hot | 0x4800 | MALIBASE));
  write_value_descriptor__address = ((unsigned int *) (0x4020 | MALIBASE));

  /* set the maximum address */
  max_address = MAX_ADDRESS_VALUE;

  /* set the job slot to be used */
  current_job_slot  = 0;
  
  printf ("Start LOOP through the entire address space from 0x%x to 0x%x with a bit shift each iteration\n", MALIBASE, max_address);
  while (((unsigned int) write_address < max_address) && (res == 0)) 
  {
#ifdef DEBUG_ON
    printf (" DBG: Run test with address offset 0x%x \n", test_address_offset_one_hot);
#endif
    res |= adv1_RunWriteTest (write_value_descriptor__address, write_address, current_job_slot, res);
#ifdef DEBUG_ON
    printf (" DBG: Test completed on address 0x%x  Result 0x%x \n", write_address, res);
#endif
#ifdef DEBUG_ON
    printf (" DBG: Advance the write address variable which is to be written to the WriteValue Job descriptor \n");
#endif
    /* write_address += address_step; */
    test_address_offset_one_hot = test_address_offset_one_hot << 1;
    write_address = ((unsigned int *) (test_address_offset_one_hot | 0x4800 | MALIBASE));

#ifdef DEBUG_ON
    printf (" DBG: NEXT write_address=0x%x \n", write_address);
#endif
    
  }
  printf ("END LOOP\n\n");
  
  Mali_clear_irqs_and_set_all_masks();
  return res;
  
}

void adv1_delay (void) {
  int n;
  int tmp, factor;
#ifdef DEBUG_ON
  factor = 128;
#else
  #ifdef MALI_MINIMUM_DELAYS
    factor = 2;
  #else
    factor = 2048;
  #endif
#endif  
  for (n=0;n<(factor*0x1);n++)
  {
    /* do something to take up time */
    tmp = tmp + mali_step;
  }
}

int Check_adv1_t720_check_0_a (struct t_adv1_t720_mem_0 *p) {
  struct t_CheckData_adv1_t720_check_0_a *r = &CheckData_adv1_t720_check_0_a;
  int res = 0;
  Mali_Message("Performing data check of 48 bytes\n");

  res |= Mali_MemCmp(p->data_00001000, 0x00001000, r->data_00001000, 0, 12);
  /* Return -1 if a compare fail */
  if (res) { return -1; };
  Mali_Message("Check okay \n");
  return 0;
};

int adv1_RunWriteTest (volatile unsigned int * write_value_descriptor__address, volatile unsigned int * write_address, int current_job_slot, int res) {
  unsigned int              tmp_value, CC_reg;
  int                       i, j, base_address, ttb_address;
  volatile unsigned int *   write_value_descriptor__status;
  
  base_address = MALIBASE;
  
#ifdef DEBUG_ON
    printf (" DBG: Start the cycle counter\n");
#endif
    Mali_WrReg (0x20, 0, GPU_COMMAND, GPU_COMMAND__CYCLE_COUNT_START);
    
#ifdef DEBUG_ON
    printf (" DBG: Wait a little while for the cycle counter to increment \n");
    adv1_delay();
#else
    j= 1000;
    while (j-- > 0)
    { 
      adv1_delay();
    }
#endif
    
#ifdef DEBUG_ON
    printf (" DBG: Setup the write address field within the WriteValue Job descriptor \n");
#endif
    *write_value_descriptor__address = (unsigned int) write_address;
    while (*write_value_descriptor__address != (unsigned int) write_address) {};
#ifdef DEBUG_ON
    printf (" DBG: WriteValue.{write address}[0x%x] = 0x%x\n", write_value_descriptor__address, *write_value_descriptor__address);
#endif
    
    
#ifdef DEBUG_ON
    printf (" DBG: Stop the cycle counter\n");
#endif
    Mali_WrReg (0x20, 0, GPU_COMMAND, GPU_COMMAND__CYCLE_COUNT_STOP);
    
#ifdef DEBUG_ON
    printf (" DBG: Wait for the cycle counter to stop \n");
    printf (" DBG: Poll the CYCLE_COUNT_ACTIVE flag until it is 0\n");
#endif
    (void) PollReg (0x20, 0x0, GPU_STATUS, 6, 0, "GPU_STATUS[CYCLE_COUNT_ACTIVE]");
    
    
#ifdef DEBUG_ON
    printf (" DBG: Sample the cycle count value when it has stopped and store in CC_reg \n");
#endif
    CC_reg = Mali_RdReg (0x20,0,CYCLE_COUNT_LO) | (Mali_RdReg (0x20,0,CYCLE_COUNT_HI) << 32);


#ifdef DEBUG_ON
    printf (" DBG: Setup the job slot\n");
#endif
    Mali_WrReg(0x20,0x0, JOB_IRQ_CLEAR       , 0xFFFFFFFF);
    Mali_WrReg(0x20,0x0, JOB_IRQ_MASK        , 0xFFFFFFFF);
    
    Mali_write_JSn_register(JSn_CONFIG_NEXT      , current_job_slot, 0x00003000); /* clean&invalidate caches on end of job */
    Mali_write_JSn_register(JSn_HEAD_NEXT_LO     , current_job_slot, (0x4000 | MALIBASE)); 
    Mali_write_JSn_register(JSn_AFFINITY_NEXT_LO , current_job_slot, 0xFFFFFFFF);
#ifdef DEBUG_ON
    printf (" DBG: Execute the job\n");
#endif
    Mali_write_JSn_register(JSn_COMMAND_NEXT     , current_job_slot, 0x00000001);
    
#ifdef DEBUG_ON
    printf (" DBG: Wait for the job to complete\n");
    printf (" DBG: Poll the JOB_IRQ_STATUS (JOB_DONE/JOB_FAIL) flags until they are none zero\n");
#endif
    while ((Mali_RdReg (0x20, 0x0, JOB_IRQ_STATUS)) == 0)
    {
    }
#ifdef DEBUG_ON
    printf (" DBG: JOB_IRQ_STATUS = 0x%x\n", Mali_RdReg (0x20, 0x0, JOB_IRQ_STATUS));
    printf (" DBG: JS%d_STATUS = 0x%x\n", current_job_slot, Mali_read_JSn_register (JSn_STATUS, current_job_slot));
    
    printf (" DBG: Read the address directly\n");
#endif
    tmp_value = *write_address;
#ifdef DEBUG_ON
    printf (" DBG: write_address=0x%x *write_address=0x%x\n", write_address, tmp_value);
#endif
    
#ifdef DEBUG_ON
    printf (" DBG: Compare the value read to CC_reg \n");
#endif
    if (CC_reg != *write_address)
    {
      printf ("ERROR: The CYCLE_COUNT register differs from the value written by the job: CC_reg=0x%x write_value=0x%x\n", CC_reg, *write_address);
      res = 1;
    }
    else
    {
      printf ("Check passed  ---> write_address=0x%x CC_reg=0x%x write_value=0x%x\n", write_address, CC_reg, *write_address);
    }

#ifdef DEBUG_ON
    printf (" DBG: Clear the job interrupts \n");
#endif
    Mali_WrReg(0x20,0x0, JOB_IRQ_CLEAR       , 0xFFFFFFFF);
    Mali_WrReg(0x20,0x0, JOB_IRQ_MASK        , 0xFFFFFFFF);

#ifdef DEBUG_ON
    printf (" DBG: Clear the status bits in the WriteValue job descriptor so it can be reused ===\n");    
#endif
    write_value_descriptor__status = (unsigned int *) (0x4000 | MALIBASE);
    for (i=0;i<4;i++)
    {
      *write_value_descriptor__status = 0;
      write_value_descriptor__status += 1;
    }
    /* DEBUG: check that the descriptor has the correct values
    write_value_descriptor__status = (unsigned int *) (base_address | 0x4000);
    printf ("Checking write_value_descriptor at 0x%x:\n  ", write_value_descriptor__status); 
    for (j=0;j<12;j++)
    {
      if ((j%4)==0 && j != 0)
      {
        printf ("\n  ");
      }
      printf ("word[%02d]=0x%08x ", j, *write_value_descriptor__status); 
      write_value_descriptor__status += 1;
    }
    printf ("\n");
    */
    return res;
}

unsigned int PollReg (unsigned int unit, unsigned int core, unsigned int regnum, unsigned int bit_location, unsigned int val, char * name_str)
{
  int regnum_value =  Mali_RdReg(0x20,0,regnum);


#ifdef DEBUG_ON
  printf (" DBG: Polling (for %d) of register %s 0x%x bit %d started\n", val, name_str, regnum, bit_location);
#endif
  
  if (val)
  {
    while ((regnum_value & (1<<bit_location)) != (1<<bit_location))
    {
      regnum_value = Mali_RdReg(0x20,0,regnum);
    }  
  }
  else
  {
    while ((regnum_value & (1<<bit_location)) == (1<<bit_location))
    {
      regnum_value = Mali_RdReg(0x20,0,regnum);
    }  
  }
#ifdef DEBUG_ON
  printf (" DBG: Polling (for %d) of register %s 0x%x bit %d ended   current value=0x%x \n", val, name_str, regnum, bit_location, regnum_value);
#endif
  
  return regnum_value;
};

